package com.ibm.xml.parsers;

import com.ibm.xml.framework.EntityPool;
import com.ibm.xml.framework.ParserState;
import com.ibm.xml.framework.StringPool;
import com.ibm.xml.framework.XMLParser;
import java.io.IOException;
import org.xml.sax.DTDHandler;
import org.xml.sax.DocumentHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class SAXParser extends XMLParser {
   private DTDHandler fDTDHandler;
   private DocumentHandler fDocumentHandler;
   protected boolean fRootElement;

   public SAXParser() {
      ParserState var1 = ((XMLParser)this).getParserState();
      var1.useDefaultStringPool();
      var1.useDefaultAttrPool();
      var1.useDefaultEntityPool();
      var1.useDefaultElementDeclPool();
      ((XMLParser)this).setDocumentHandler(this);
      ((XMLParser)this).setDocumentTypeHandler(this);
   }

   public void setEntityResolver(EntityResolver var1) {
      ((XMLParser)this).getEntityHandler().setEntityResolver(var1);
   }

   public void setDTDHandler(DTDHandler var1) {
      this.fDTDHandler = var1;
   }

   public void setDocumentHandler(DocumentHandler var1) {
      this.fDocumentHandler = var1;
   }

   public void doctypeDecl(int var1) throws Exception {
   }

   public void startInternalSubset() throws Exception {
   }

   public void endInternalSubset() throws Exception {
   }

   public void startExternalSubset(int var1, int var2) throws Exception {
   }

   public void endExternalSubset() throws Exception {
   }

   public void elementDecl(int var1) throws Exception {
   }

   public void attlistDecl(int var1, int var2) throws Exception {
   }

   public void internalEntityDecl(int var1) throws Exception {
   }

   public void externalEntityDecl(int var1) throws Exception {
   }

   public void unparsedEntityDecl(int var1) throws Exception {
      if (this.fDTDHandler != null) {
         ParserState var2 = ((XMLParser)this).getParserState();
         EntityPool var3 = var2.getEntityPool();
         StringPool var4 = var2.getStringPool();
         String var5 = var4.toString(var3.getEntityName(var1));
         String var6 = var4.toString(var3.getPublicId(var1));
         String var7 = var4.toString(var3.getSystemId(var1));
         String var8 = var4.toString(var3.getNotationName(var1));
         this.fDTDHandler.unparsedEntityDecl(var5, var6, var7, var8);
      }

   }

   public void parameterEntityDecl(int var1) {
   }

   public void notationDecl(int var1) throws Exception {
      if (this.fDTDHandler != null) {
         ParserState var2 = ((XMLParser)this).getParserState();
         EntityPool var3 = var2.getEntityPool();
         StringPool var4 = var2.getStringPool();
         String var5 = var4.toString(var3.getNotationName(var1));
         String var6 = var4.toString(var3.getPublicId(var1));
         String var7 = var4.toString(var3.getSystemId(var1));
         this.fDTDHandler.notationDecl(var5, var6, var7);
      }

   }

   public boolean sendCharDataAsCharArray() {
      return true;
   }

   public void startDocument(int var1, int var2, int var3) throws Exception {
      StringPool var4 = ((XMLParser)this).getParserState().getStringPool();
      var4.orphanString(var1);
      var4.orphanString(var2);
      var4.orphanString(var3);
      if (this.fDocumentHandler != null) {
         this.fDocumentHandler.setDocumentLocator(((XMLParser)this).getLocator());
         this.fDocumentHandler.startDocument();
      }

   }

   public void endDocument() throws Exception {
      if (this.fDocumentHandler != null) {
         this.fDocumentHandler.endDocument();
      }

   }

   public void startElement(int var1, int var2) throws Exception {
      this.fRootElement = true;
      if (this.fDocumentHandler != null) {
         ParserState var3 = ((XMLParser)this).getParserState();
         this.fDocumentHandler.startElement(var3.getStringPool().toString(var1), var3.getAttrPool().getAttributeList(var2));
      }

      ((XMLParser)this).getParserState().getAttrPool().releaseAttrList(var2);
   }

   public void endElement(int var1) throws Exception {
      if (this.fDocumentHandler != null) {
         this.fDocumentHandler.endElement(((XMLParser)this).getParserState().getStringPool().toString(var1));
      }

   }

   public void startEntityReference(int var1) throws Exception {
   }

   public void endEntityReference(int var1) throws Exception {
   }

   public void characters(int var1, boolean var2) throws Exception {
   }

   public void ignorableWhitespace(int var1, boolean var2) throws Exception {
   }

   public void processingInstruction(int var1, int var2) throws Exception {
      StringPool var3 = ((XMLParser)this).getParserState().getStringPool();
      String var4 = var3.orphanString(var1);
      String var5 = var3.orphanString(var2);
      if (this.fDocumentHandler != null) {
         this.fDocumentHandler.processingInstruction(var4, var5);
      }

   }

   public void comment(int var1) throws Exception {
      ((XMLParser)this).getParserState().getStringPool().releaseString(var1);
   }

   public void characters(char[] var1, int var2, int var3, boolean var4) throws Exception {
      if (this.fDocumentHandler != null) {
         this.fDocumentHandler.characters(var1, var2, var3);
      }

   }

   public void ignorableWhitespace(char[] var1, int var2, int var3, boolean var4) throws Exception {
      if (this.fDocumentHandler != null) {
         this.fDocumentHandler.ignorableWhitespace(var1, var2, var3);
      }

   }

   public void parse(InputSource var1) throws SAXException, IOException {
      this.fRootElement = false;
      super.parse(var1);
      if (!this.fRootElement) {
         try {
            ((XMLParser)this).error(118);
         } catch (Exception var3) {
            throw new SAXException(var3);
         }
      }
   }
}
